@@ -20,10 +20,23 @@
 # MA 02110-1301, USA.
 
 package Lintian::binaries;
+
 use strict;
 use warnings;
 use autodie;
 
+use File::Spec;
+use List::MoreUtils qw(any);
+use Moo;
+
+use Lintian::Data;
+use Lintian::Relation qw(:constants);
+use Lintian::Spelling qw(check_spelling spelling_tag_emitter);
+use Lintian::Tags qw(tag);
+use Lintian::Util qw(internal_error strip);
+
+with('Lintian::Check');
+
 use constant NUMPY_REGEX => qr/
     \Qmodule compiled against ABI version \E (?:0x)?%x
     \Q but this version of numpy is \E (?:0x)?%x
@@ -39,15 +52,6 @@ use constant GUILE_PATH_REGEX => qr,^usr/lib/[^/]+/[^/]+/guile/[^/]+/.+\.go$,o;
 #   not strippable for static libs.  Accordingly, it is omitted below.
 use constant DEBUG_SECTIONS => qw(.debug_info .zdebug_info);
 
-use File::Spec;
-use List::MoreUtils qw(any);
-
-use Lintian::Check qw(check_spelling spelling_tag_emitter);
-use Lintian::Data;
-use Lintian::Relation qw(:constants);
-use Lintian::Tags qw(tag);
-use Lintian::Util qw(internal_error strip);
-
 my $ARCH_REGEX = Lintian::Data->new('binaries/arch-regex', qr/\s*\~\~/o,
     sub { return qr/$_[1]/ });
 my $ARCH_64BIT_EQUIVS
@@ -114,10 +118,16 @@ our $OBSOLETE_CRYPT_FUNCTIONS
 
 our $ARCH_32_REGEX;
 
-sub run {
-    my ($pkg, $type, $info, $proc, $group) = @_;
+sub always {
+    my ($self) = @_;
 
-    my ($madir, %directories, $built_with_golang, %SONAME);
+    my $pkg = $self->package;
+    my $type = $self->type;
+    my $info = $self->info;
+    my $proc = $self->processable;
+    my $group = $self->group;
+
+    my ($madir, %directories, $built_with_golang, $built_with_octave, %SONAME);
     my ($arch_hardening, $gnu_triplet_re, $ruby_triplet_re);
     my $needs_libc = '';
     my $needs_libcxx = '';
@@ -139,10 +149,13 @@ sub run {
 
     my $src = $group->get_source_processable;
     if (defined($src)) {
-        $built_with_golang = $src->info->relation('build-depends')
+        $built_with_golang = $src->info->relation('build-depends-all')
           ->implies('golang-go | golang-any');
+        $built_with_octave
+          = $src->info->relation('build-depends')->implies('dh-octave');
     } else {
         $built_with_golang = $pkg =~ m/^golang-/;
+        $built_with_octave = $pkg =~ m/^octave-/;
     }
 
     foreach my $file (sort keys %{$info->objdump_info}) {
@@ -597,7 +610,9 @@ sub run {
                     $needs_libcxx_count++;
                 }
             }
-            if ($no_libc and not $fname =~ m,/libc\b,) {
+            if (    $no_libc
+                and not $fname =~ m,/libc\b,
+                and not($built_with_octave and $fname =~ m/\.(oct|mex)$/)) {
                 # If there is no libc dependency, then it is most likely a
                 # bug.  The major exception is that some C++ libraries,
                 # but these tend to link against libstdc++ instead.  (see
