@@ -23,18 +23,24 @@
 # MA 02110-1301, USA.
 
 package Lintian::scripts;
+
 use strict;
 use warnings;
 use autodie;
 
-use POSIX qw(strftime);
+use Capture::Tiny qw(capture);
 use List::MoreUtils qw(any);
+use Moo;
+use POSIX qw(strftime);
+use Try::Tiny;
 
-use Lintian::Check qw($known_shells_regex);
+use Lintian::Spelling qw($known_shells_regex);
 use Lintian::Data;
 use Lintian::Relation;
 use Lintian::Tags qw(tag);
-use Lintian::Util qw(do_fork internal_error strip);
+use Lintian::Util qw(internal_error safe_qx strip);
+
+with('Lintian::Check');
 
 # This is a map of all known interpreters.  The key is the interpreter
 # name (the binary invoked on the #! line).  The value is an anonymous
@@ -236,8 +242,11 @@ sub script_tag {
     return;
 }
 
-sub run {
-    my ($pkg, undef, $info) = @_;
+sub binary {
+    my ($self) = @_;
+
+    my $pkg = $self->package;
+    my $info = $self->info;
 
     my (%executable, %ELF, %scripts, %seen_helper_cmds);
 
@@ -292,6 +301,12 @@ sub run {
 
         my ($base) = $interpreter =~ m,([^/]*)$,;
 
+        # Ignore Python scripts that are shipped under dist-packages; these
+        # files aren't supposed to be called as scripts.
+        next
+          if $base eq 'python'
+          and $filename =~ m,^usr/lib/python3/dist-packages/,;
+
         # allow exception for .in files that have stuff like #!@PERL@
         next
           if (  $filename =~ m,\.in$,
@@ -1368,16 +1383,8 @@ sub script_is_evil_and_wrong {
 # -n option to check syntax, discarding output and returning the exit status.
 sub check_script_syntax {
     my ($interpreter, $path) = @_;
-    my $fs_path = $path->fs_path;
-    my $pid = do_fork();
-    if ($pid == 0) {
-        open(STDOUT, '>', '/dev/null');
-        open(STDERR, '>&', \*STDOUT);
-        exec $interpreter, '-n', $fs_path
-          or internal_error("cannot exec $interpreter: $!");
-    } else {
-        waitpid $pid, 0;
-    }
+
+    safe_qx($interpreter, '-n', $path->fs_path);
     return $?;
 }
 
